@@ -56,12 +56,17 @@
 //! original source file.  (a.k.a., it’s incremental!)
 
 pub mod arena;
+pub mod assert;
 pub mod c;
 pub mod cycles;
 #[macro_use]
 mod debugging;
 pub mod graph;
+#[cfg(feature = "json")]
+pub mod json;
 pub mod partial;
 pub mod paths;
 pub mod stitching;
 pub(crate) mod utils;
+#[cfg(feature = "json")]
+pub mod visualization;
